package cbg.spy.inspectors;

import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.ToolBar;


public class InspectorService implements Inspector {
	protected static InspectorService soleInstance = new InspectorService();
	
	protected SpyEvent spyOn;
	
	protected static void inspect(MouseEvent e) {
		Control control = e.display.getCursorControl();
		Point global = e.display.getCursorLocation();
		if(control == null) return;
		Point local = control.toControl(global);
		Item item = findControl(control, local);
		InspectorService.getSoleInstance().inspect(new SpyEvent(control, item, e, local));
	}
	/* Answer the "real" control that was hit. This is needed for Items because
	 * the Item manager performs the hit-detection not the item itself. For example,
	 * a toolbar item does not receive hit detection, the toolbar does.
	 */ 
	private static Item findControl(Control control, Point local) {
		if(control instanceof ToolBar) {
			return ((ToolBar)control).getItem(local);
		} else if(control instanceof CoolBar) {
			CoolBar bar = (CoolBar)control;
			CoolItem[] items = bar.getItems();
			for (int i = 0; i < items.length; i++) {
				CoolItem item = items[i];
				if(item.getBounds().contains(local)) return item;
			}
		}
		return null;
	}

	public InspectorService() {}
	public static void addHooksTo(final Control root, final Control control, final Runnable execute) {
		control.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				try {
					control.setCapture(false);
					if(control.getDisplay().getCursorControl() == control) return;
					inspect(e);
					execute.run();
				} finally {
					if(root != null) root.setCursor(null);
				}
			}
		});
	}



	public void inspect(SpyEvent event) {
		setSpyOn(event);
	}
	public static InspectorService getSoleInstance() {
		return soleInstance;
	}
	public void setSpyOn(SpyEvent spyOn) {
		this.spyOn = spyOn;
	}

	public SpyEvent getSpyOn() {
		return spyOn;
	}

}
