package cbg.spy.views;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import cbg.spy.SpyPlugin;
import cbg.spy.Util;
import cbg.spy.inspectors.InspectorService;
import cbg.spy.inspectors.SpyEvent;
/**
 * The view to show the containment tree.
 */
public class CSpy extends ViewPart {
	private Action action1;
	protected Cursor waitCursor;
	protected Object lastInput;
	protected TreeViewer tree;
	protected Thread updateThread;
	protected static Object[] EMPTY = new Object[0];
	protected List commands;
	protected Object commandLock = new Object();
	private boolean ignoreSelection;
	class ControlContentProvider implements ITreeContentProvider {
		protected Map childMap;
		public ControlContentProvider() {
			childMap = new HashMap();
			fillMethodMap();
		}
		private void fillMethodMap() {
			String GET_ITEMS = "getItems";
			childMap.put(org.eclipse.swt.widgets.Tree.class, GET_ITEMS);
			childMap.put(TabFolder.class, GET_ITEMS);
			childMap.put(Table.class, GET_ITEMS);
			childMap.put(Table.class, GET_ITEMS);
			childMap.put(Combo.class, GET_ITEMS);
			childMap.put(ToolBar.class, GET_ITEMS);
			childMap.put(CoolBar.class, GET_ITEMS);
		}
		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof Composite) {
				return childrenOfComposite((Composite) parentElement);
			}
			return EMPTY;
		}
		protected Object[] childrenOfComposite(Composite composite) {
			Object lookup = childMap.get(composite.getClass());
			Method method = null;
			if (lookup instanceof String) {
				String methodName = (String) lookup;
				try {
					method = composite.getClass().getMethod(methodName, null);
					childMap.put(composite.getClass(), method);
				} catch (NoSuchMethodException e) {
					e.printStackTrace();
					return EMPTY;
				}
			} else {
				method = (Method) lookup;
			}
			if (method == null) {
				return composite.getChildren();
			}
			try {
				return (Object[]) method.invoke(composite, null);
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
			return EMPTY;
		}
		public Object getParent(Object element) {
			if (element instanceof Control) {
				return ((Control) element).getParent();
			} else if(element instanceof ToolItem) {
				return ((ToolItem)element).getParent();
			} else if(element instanceof CoolItem) {
				return ((CoolItem)element).getParent();
			} else if (element instanceof TabItem) {
				return ((TabItem)element).getParent();
			}
			return null;
		}
		public boolean hasChildren(Object element) {
			return getChildren(element).length > 0;
		}
		public Object[] getElements(Object inputElement) {
			return getChildren(inputElement);
		}
		public void dispose() {
		}
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}
	class ControlLabelProvider implements ILabelProvider {
		protected Map printMap;
		public ControlLabelProvider() {
			printMap = new HashMap();
			fillPrintMap();
		}
		protected void fillPrintMap() {
			//			printMap.put(org.eclipse.swt.widgets.Tree.class, GET_ITEMS);
			//			printMap.put(TabFolder.class, GET_ITEMS);
			//			printMap.put(Table.class, GET_ITEMS);
			//			printMap.put(Table.class, GET_ITEMS);
			//			printMap.put(Combo.class, GET_ITEMS);
			//			printMap.put(ToolBar.class, GET_ITEMS);
			//			printMap.put(CoolBar.class, GET_ITEMS);
		}
		public Image getImage(Object element) {
			return null;
		}
		public String getText(Object element) {
			return Util.shortName(element);
		}
		public void addListener(ILabelProviderListener listener) {
		}
		public void dispose() {
		}
		public boolean isLabelProperty(Object element, String property) {
			return false;
		}
		public void removeListener(ILabelProviderListener listener) {
		}
	}
	/**
	 * The constructor.
	 */
	public CSpy() {
		commands = new LinkedList();
		createThread();
	}
	private void createThread() {
		updateThread = new Thread(new Runnable() {
			public void run() {
				while (true) {
					List clone = null;
					synchronized (commandLock) {
						try {
							if(commands.size() == 0) commandLock.wait();
						} catch (InterruptedException e) {
						}
						if (commands.size() > 0) {
							clone = new ArrayList(commands);
							commands.clear();
						}
					}
					if (clone != null) {
						for (Iterator iter = clone.iterator(); iter.hasNext();) {
							Runnable command = (Runnable) iter.next();
							command.run();
						}
					}
				}
			}
		});
		updateThread.setName("cbg-updater");
		updateThread.setDaemon(true);
		updateThread.start();
	}
	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		tree = new TreeViewer(parent);
		tree.setContentProvider(new ControlContentProvider());
		tree.setLabelProvider(new ControlLabelProvider());
		tree.setInput(InspectorService.getSoleInstance().getSpyOn());
		tree.expandAll();
		
		//		FormData data = new FormData();
		//		data.top = new FormAttachment(0, 0);
		//		data.left = new FormAttachment(0, 0);
		//		data.right = new FormAttachment(100, 0);
		//		data.bottom = new FormAttachment(100, 0);
		//		tree.getControl().setLayoutData(data);
		makeActions();
		hookContextMenu();
		contributeToActionBars();
		hookListeners();
		getSite().setSelectionProvider(new SimpleSelectionProvider());
	}
	private void hookListeners() {
		/* Listen to selection changes from myself. */
		getSite().getWorkbenchWindow().getSelectionService().addSelectionListener(new ISelectionListener() {
			public void selectionChanged(IWorkbenchPart part, ISelection selection) {
				handleSelectionChanged(part, selection);
			}
		});
		tree.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				handleItemSelected(event);
			}
		});
		Runnable doBlock = new Runnable() {
			public void run() {
				getViewSite().getSelectionProvider().setSelection(
					new StructuredSelection(InspectorService.getSoleInstance().getSpyOn()));
			}
		};
		InspectorService.addHooksTo(getRoot(), tree.getControl(), doBlock);
	}
	protected void handleItemSelected(SelectionChangedEvent event) {
		if(ignoreSelection) return;
		Object selected = ((IStructuredSelection) event.getSelection()).getFirstElement();
		GC gc = null;
		try {
			Runnable command = null;
			gc = highlightControl(selected);
			command = newRefreshControl(selected);
			if(command != null) {
				synchronized (commandLock) {
					commands.add(command);
					commandLock.notify();
				}
			}
		} finally {
			if (gc != null)
				gc.dispose();
		}
	}
	protected Cursor getWaitCursor() {
		if(waitCursor == null) {
			waitCursor = new Cursor(tree.getControl().getDisplay(), SWT.CURSOR_CROSS);
		}
		return waitCursor;
	}
	protected Control getRoot() {
		return getSite().getWorkbenchWindow().getShell();
	}
	private Runnable newRefreshControl(final Object control) {
		return new Runnable() {
			public void run() {
				try {
					Thread.sleep(2000);
				} catch (InterruptedException e) {
				}
				tree.getControl().getDisplay().asyncExec(new Runnable() {
					public void run() {
						unhighlightControl(control);
					}
				});
			}
		};
	}

	protected void handleSelectionChanged(IWorkbenchPart part, ISelection selection) {
		if (part instanceof CSpy) {
			SpyEvent spyEvent = (SpyEvent)((IStructuredSelection) selection).getFirstElement();
			Control widget = (Control)spyEvent.control; 
			while (widget != null && !(widget instanceof Composite)) {
				widget = widget.getParent();
			}
			Object newInput = widget.getParent() == null ? widget : widget.getParent();
			if(!newInput.equals(lastInput)) {
				lastInput = newInput;
				tree.setInput(newInput);
			}
			ignoreSelection = true;
			Object select = spyEvent.item == null ? spyEvent.control : spyEvent.item;
			tree.setSelection(new StructuredSelection(select), true);
			ignoreSelection = false;
		}
	}
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				CSpy.this.fillContextMenu(manager);
			}
		});
		//		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		//		viewer.getControl().setMenu(menu);
		//		getSite().registerContextMenu(menuMgr, viewer);
	}
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}
	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(action1);
		manager.add(new Separator());
	}
	protected void fillContextMenu(IMenuManager manager) {
		manager.add(action1);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator("Additions"));
	}
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(action1);
	}
	private void makeActions() {
		action1 = new Action() {
			public void run() {
				getRoot().setCursor(getWaitCursor());
				tree.getControl().setCapture(true);
			}
		};
		action1.setText("Spy");
		action1.setToolTipText("Spy");
		action1.setImageDescriptor(SpyPlugin.getImageDescriptor("magnifyingGlass4.gif"));
	}
	protected void unhighlightControl(Object object) {
		Control control = null;
		if(object instanceof Item) {
			control = getParentOf((Item)object);
		} else if(object instanceof Control) {
			control = (Control)object;
		}
		if(control != null && !control.isDisposed()) {
			control.redraw(0, 0, control.getSize().x, control.getSize().y, true);
			control.update();
			control.getParent().update();
		}
	}
	private Control getParentOf(Item item) {
		if (item instanceof TableItem) {
			return ((TableItem)item).getParent();
		} else if(item instanceof TreeItem) {
			return ((TreeItem)item).getParent();
		} else if(item instanceof ToolItem) {
			return ((ToolItem)item).getParent();
		} else if(item instanceof CoolItem) {
			return ((CoolItem)item).getParent();
		}
		return null;
	
	}
	protected GC highlightControl(Object object) {
		GC gc = new GC(Display.getDefault());
		int line = 3;
		gc.setLineWidth(line);
		Control control = object instanceof Item ? getParentOf((Item)object) : (Control)object;
		if(control == null) return null;
		Rectangle parentBounds = new Rectangle(0, 0, control.getSize().x, control.getSize().y);
		Rectangle childBounds = parentBounds; 
		if(object instanceof Item) {
			childBounds = getBoundsOf((Item)object);
		}
		gc.setForeground(control.getDisplay().getSystemColor(SWT.COLOR_BLACK));
		gc.setBackground(control.getDisplay().getSystemColor(SWT.COLOR_BLACK));
		Point origin = control.toDisplay(new Point(0, 0));
		Point offset = origin;
		if(object instanceof Item)
			offset = control.toDisplay(new Point(childBounds.x, childBounds.y));
		Point size = new Point(parentBounds.width, parentBounds.height);
		gc.setClipping(origin.x, origin.y, size.x + line, size.y);
		gc.drawRectangle(offset.x + 1, offset.y + 1, childBounds.width - line, childBounds.height - line);
		System.out.println("clip: " + gc.getClipping() + " " + "offset: " + offset);
		return gc;
	}
	private Rectangle getBoundsOf(Item item) {
		if (item instanceof TableItem) {
			return ((TableItem)item).getBounds(0);
		} else if(item instanceof TreeItem) {
			return ((TreeItem)item).getBounds();
		} else if(item instanceof ToolItem) {
			return ((ToolItem)item).getBounds();
		} else if(item instanceof CoolItem) {
			return ((CoolItem)item).getBounds();
		}
		return null;
	}
	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		tree.getControl().setFocus();
	}
	public void dispose() {
		super.dispose();
		if(waitCursor != null) waitCursor.dispose();
	}

}