package cbg.spy.views;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.internal.presentations.ISelfUpdatingAction;
import org.eclipse.ui.part.ViewPart;

import cbg.spy.SpyPlugin;
import cbg.spy.inspectors.CLabelInspector;
import cbg.spy.inspectors.Inspector;
import cbg.spy.inspectors.InspectorMap;
import cbg.spy.inspectors.InspectorService;
import cbg.spy.inspectors.SpyEvent;
import cbg.spy.inspectors.ToolbarInspector;
import cbg.spy.inspectors.ViewPaneInspector;
/**
 * This sample class demonstrates how to plug-in a new
 * workbench view. The view shows data obtained from the
 * model. The sample creates a dummy model on the fly,
 * but a real implementation would connect to the model
 * available either in this or another plug-in (e.g. the workspace).
 * The view is connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model
 * objects should be presented in the view. Each
 * view can present the same model objects using
 * different labels and icons, if needed. Alternatively,
 * a single label provider can be shared between views
 * in order to ensure that objects of the same type are
 * presented in the same way everywhere.
 * <p>
 */
public class Info extends ViewPart {
	/**
	 * Internal document listener.
	 */
	class InternalDocumentListener implements IDocumentListener {
		/**
		 * @see IDocumentListener#documentAboutToBeChanged(DocumentEvent)
		 */
		public void documentAboutToBeChanged(DocumentEvent e) {
		}
		
		/**
		 * @see IDocumentListener#documentChanged(DocumentEvent)
		 */
		public void documentChanged(DocumentEvent e) {
			IDocument doc = textViewer.getDocument();
			if (doc == null) return;
			revealEndOfDocument();
			//updateStyleRanges(doc);
		}
	}
	protected ISelection lastSelection;
	protected InspectorMap inspectorMap;
	protected Map globalActions= new HashMap(10);
	protected List selectionActions = new ArrayList(3);
	protected ISelectionProvider selectionProvider;
	protected TextViewer textViewer;
	private Action clearAction;
	/**
	 * The constructor.
	 */
	public Info() {
		inspectorMap = new InspectorMap();
		addDefaultInspectors();
	}
	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		textViewer = new TextViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL);
		IDocument doc = new Document("");
		doc.addDocumentListener(new InternalDocumentListener());
		textViewer.setDocument(doc);
		textViewer.activatePlugins();
		textViewer.getSelectionProvider().addSelectionChangedListener(getSelectionChangedListener());
		textViewer.setEditable(false);
		makeActions();
		hookContextMenu();
		contributeToActionBars();
		hookListeners();
	}
	protected ISelectionChangedListener getSelectionChangedListener() {
		return new ISelectionChangedListener() {
				public void selectionChanged(SelectionChangedEvent event) {
					updateSelectionDependentActions();
				}
			};
	}
	private void hookListeners() {
		getSite().getWorkbenchWindow().getSelectionService().addSelectionListener(new ISelectionListener() {
			public void selectionChanged(IWorkbenchPart part, ISelection selection) {
				if (part instanceof CSpy && !selection.equals(lastSelection)) {
					handleNewSpyOn(selection);
					lastSelection = selection;
				}
			}
		});
	}
	protected void handleNewSpyOn(ISelection selection) {
		inspect(InspectorService.getSoleInstance().getSpyOn());
	}
	public void inspect(SpyEvent spyEvent) {
		Iterator inspectors = inspectorMap.iteratorFor(spyEvent.control);
		if (inspectors == null)
			return;
		while (inspectors.hasNext()) {
			Inspector inspector = (Inspector) inspectors.next();
			inspector.inspect(spyEvent);
		}
	}
	private void addDefaultInspectors() {
		inspectorMap.addInspector(org.eclipse.swt.custom.CLabel.class.getName(), new CLabelInspector(this));
		inspectorMap.addInspector(org.eclipse.swt.widgets.ToolBar.class.getName(), new ToolbarInspector(this));
		inspectorMap.addInspector(org.eclipse.ui.internal.ViewPane.class.getName(), new ViewPaneInspector(this));
	}
	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(textViewer.getControl());
		textViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, textViewer);
	}
	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}
	private void fillLocalPullDown(IMenuManager manager) {
//		manager.add(action1);
//		manager.add(new Separator());
//		manager.add(action2);
	}
	protected void fillContextMenu(IMenuManager manager) {
		// Read-only
		manager.add((IAction)globalActions.get(ActionFactory.COPY.getId()));
		manager.add((IAction)globalActions.get(ActionFactory.SELECT_ALL.getId()));
		
		// Writable
//		updateAction(ITextEditorActionConstants.PASTE);
//		manager.add((IAction)globalActions.get(ITextEditorActionConstants.CUT));
//		manager.add((IAction)globalActions.get(ITextEditorActionConstants.COPY));
//		manager.add((IAction)globalActions.get(ITextEditorActionConstants.PASTE));
//		manager.add((IAction)globalActions.get(ITextEditorActionConstants.SELECT_ALL));
		// Other plug-ins can contribute there actions here
		manager.add(new Separator("Additions"));
		manager.add(clearAction);
	}
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(clearAction);
//		manager.add(action2);
	}
	public void println(Object object) {
		append("" + object + "\n");
	}
	public void append(Object object) {
		IDocument doc = textViewer.getDocument();
		String text = "" + object;
		try {
			textViewer.getDocument().replace(doc.getLength(), 0, text);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}
	private void makeActions() {
		clearAction = new Action() {
			public void run() {
				textViewer.getDocument().set("");
			}
		};
		clearAction.setText("Clea&r");
		clearAction.setToolTipText("Clear");
		clearAction.setImageDescriptor(SpyPlugin.getImageDescriptor("clear_co.gif"));
		// In order for the clipboard actions to accessible via their shortcuts
		// (e.g., Ctrl-C, Ctrl-V), we *must* set a global action handler for
		// each action		
		IActionBars actionBars= getViewSite().getActionBars();
		TextViewerAction action= new TextViewerAction(textViewer, TextViewer.CUT);
		action.configureAction("Cu&t@Ctrl+X", "Cut", "Cut");
		setGlobalAction(actionBars, ActionFactory.CUT.getId(), action);
		action= new TextViewerAction(textViewer, TextViewer.COPY);
		action.configureAction("&Copy@Ctrl+C", "Copy", "Copy");
		setGlobalAction(actionBars, ActionFactory.COPY.getId(), action);
		action= new TextViewerAction(textViewer, TextViewer.PASTE);
		action.configureAction("&Paste@Ctrl+V", "Paste", "Paste");
		setGlobalAction(actionBars, ActionFactory.PASTE.getId(), action);
		action= new TextViewerAction(textViewer, TextViewer.SELECT_ALL);
		action.configureAction("Select &All@Ctrl+A", "Select All", "Select All");
		setGlobalAction(actionBars, ActionFactory.SELECT_ALL.getId(), action);

		selectionActions.add(ActionFactory.CUT);
		selectionActions.add(ActionFactory.COPY);
		selectionActions.add(ActionFactory.PASTE);
		actionBars.updateActionBars();
	}
	protected void updateSelectionDependentActions() {
		Iterator iterator= selectionActions.iterator();
		while (iterator.hasNext()) {
			updateAction((ActionFactory)iterator.next());		
		}
	}

	protected void updateAction(ActionFactory actionFactory) {
		IAction action= (IAction)globalActions.get(actionFactory.getId());
		if (action instanceof ISelfUpdatingAction) {
			((ISelfUpdatingAction) action).update();
		}
	}
	protected void setGlobalAction(IActionBars actionBars, String actionID, IAction action) {
		globalActions.put(actionID, action); 
		actionBars.setGlobalActionHandler(actionID, action);
	}
	/**
	 * Reveals (makes visible) the end of the current document
	 */
	protected void revealEndOfDocument() {
		IDocument doc= textViewer.getDocument();
		int docLength= doc.getLength();
		if (docLength > 0) {
			StyledText widget= textViewer.getTextWidget();
			widget.setCaretOffset(docLength);
			widget.showSelection();
		}
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		//button.setFocus();
	}
}