package cbg.spy.views;

import org.eclipse.jface.util.ListenerList;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;

import cbg.spy.inspectors.InspectorService;

public class SimpleSelectionProvider implements ISelectionProvider {
		/**
		 * Notifies any selection changed listeners that the viewer's selection has changed.
		 * Only listeners registered at the time this method is called are notified.
		 *
		 * @param event a selection changed event
		 *
		 * @see ISelectionChangedListener#selectionChanged
		 */
		protected void fireSelectionChanged(SelectionChangedEvent event) {
			Object[] listeners = selectionChangedListeners.getListeners();
			for (int i = 0; i < listeners.length; ++i) {
				((ISelectionChangedListener) listeners[i]).selectionChanged(event);
			}
		}
		private ListenerList selectionChangedListeners = new ListenerList(3);
		
		public void addSelectionChangedListener(ISelectionChangedListener listener) {
			selectionChangedListeners.add(listener);
		}
	
		public ISelection getSelection() {
			if(InspectorService.getSoleInstance().getSpyOn() == null) return null;
			return new StructuredSelection(InspectorService.getSoleInstance().getSpyOn());
		}
	
		public void removeSelectionChangedListener(ISelectionChangedListener listener) {
			selectionChangedListeners.remove(listener);
		}
	
		public void setSelection(ISelection selection) {
			SelectionChangedEvent event = new SelectionChangedEvent(this, selection);
			fireSelectionChanged(event);
		}
	}
